@@ -47,8 +47,9 @@
 // try to detect XX16 vs XX32 vs XX64 size mismatches
 **************************************************************************/
 
-#if 0 // permissive version using "void *"
+#if !(DEBUG)
 
+// permissive version using "void *"
 #define REQUIRE_XE16 /*empty*/
 #define REQUIRE_XE24 /*empty*/
 #define REQUIRE_XE32 /*empty*/
@@ -68,20 +69,22 @@ struct LE16;
 struct LE32;
 struct LE64;
 
+namespace bele_detail {
+
 // Note:
 //   void is explicitly allowed (but there is no automatic pointer conversion because of template!)
 //   char is explicitly allowed
 //   byte is explicitly allowed
 template <class T>
-static inline constexpr bool is_xe16_type =
+inline constexpr bool is_xe16_type =
     upx::is_same_any_v<T, void, char, byte, upx_int16_t, upx_uint16_t, BE16, LE16>;
 template <class T>
-static inline constexpr bool is_xe24_type = upx::is_same_any_v<T, void, char, byte>;
+inline constexpr bool is_xe24_type = upx::is_same_any_v<T, void, char, byte>;
 template <class T>
-static inline constexpr bool is_xe32_type =
+inline constexpr bool is_xe32_type =
     upx::is_same_any_v<T, void, char, byte, upx_int32_t, upx_uint32_t, BE32, LE32>;
 template <class T>
-static inline constexpr bool is_xe64_type =
+inline constexpr bool is_xe64_type =
     upx::is_same_any_v<T, void, char, byte, upx_int64_t, upx_uint64_t, BE64, LE64>;
 
 template <class T>
@@ -93,10 +96,12 @@ using enable_if_xe32 = std::enable_if_t<is_xe32_type<T>, T>;
 template <class T>
 using enable_if_xe64 = std::enable_if_t<is_xe64_type<T>, T>;
 
-#define REQUIRE_XE16 template <class XE16, class = enable_if_xe16<XE16> >
-#define REQUIRE_XE24 template <class XE24, class = enable_if_xe24<XE24> >
-#define REQUIRE_XE32 template <class XE32, class = enable_if_xe32<XE32> >
-#define REQUIRE_XE64 template <class XE64, class = enable_if_xe64<XE64> >
+} // namespace bele_detail
+
+#define REQUIRE_XE16 template <class XE16, class = bele_detail::enable_if_xe16<XE16> >
+#define REQUIRE_XE24 template <class XE24, class = bele_detail::enable_if_xe24<XE24> >
+#define REQUIRE_XE32 template <class XE32, class = bele_detail::enable_if_xe32<XE32> >
+#define REQUIRE_XE64 template <class XE64, class = bele_detail::enable_if_xe64<XE64> >
 
 #endif // permissive version
 
@@ -152,11 +157,15 @@ static forceinline void set_ne64(XE64 *p, upx_uint64_t vv) noexcept {
 ACC_COMPILE_TIME_ASSERT_HEADER(sizeof(long) == 4)
 
 // unfortunately *not* constexpr with current MSVC
-static forceinline unsigned bswap16(unsigned v) noexcept {
+static forceinline /*constexpr*/ unsigned bswap16(unsigned v) noexcept {
     return (unsigned) _byteswap_ulong(v << 16);
 }
-static forceinline unsigned bswap32(unsigned v) noexcept { return (unsigned) _byteswap_ulong(v); }
-static forceinline upx_uint64_t bswap64(upx_uint64_t v) noexcept { return _byteswap_uint64(v); }
+static forceinline /*constexpr*/ unsigned bswap32(unsigned v) noexcept {
+    return (unsigned) _byteswap_ulong(v);
+}
+static forceinline /*constexpr*/ upx_uint64_t bswap64(upx_uint64_t v) noexcept {
+    return _byteswap_uint64(v);
+}
 
 #else
 
@@ -266,21 +275,27 @@ inline void set_le24(XE24 *p, unsigned v) noexcept {
     b[2] = ACC_ICONV(byte, (v >> 16) & 0xff);
 }
 
-inline unsigned get_le26(const void *p) noexcept { return get_le32(p) & 0x03ffffff; }
-inline unsigned get_le19_5(const void *p) noexcept { return (get_le32(p) >> 5) & 0x0007ffff; }
-inline unsigned get_le14_5(const void *p) noexcept { return (get_le32(p) >> 5) & 0x00003fff; }
+REQUIRE_XE32
+inline unsigned get_le26(const XE32 *p) noexcept { return get_le32(p) & 0x03ffffff; }
+REQUIRE_XE32
+inline unsigned get_le19_5(const XE32 *p) noexcept { return (get_le32(p) >> 5) & 0x0007ffff; }
+REQUIRE_XE32
+inline unsigned get_le14_5(const XE32 *p) noexcept { return (get_le32(p) >> 5) & 0x00003fff; }
 
-inline void set_le26(void *p, unsigned v) noexcept {
+REQUIRE_XE32
+inline void set_le26(XE32 *p, unsigned v) noexcept {
     // preserve the top 6 bits
     //   set_le32(p, (get_le32(p) & 0xfc000000) | (v & 0x03ffffff));
     // optimized version, saving a runtime bswap32
     set_ne32(p, (get_ne32(p) & ne32_to_le32(0xfc000000)) |
                     (ne32_to_le32(v) & ne32_to_le32(0x03ffffff)));
 }
-inline void set_le19_5(void *p, unsigned v) noexcept {
+REQUIRE_XE32
+inline void set_le19_5(XE32 *p, unsigned v) noexcept {
     set_le32(p, (get_le32(p) & 0xff00001f) | ((v & 0x0007ffff) << 5));
 }
-inline void set_le14_5(void *p, unsigned v) noexcept {
+REQUIRE_XE32
+inline void set_le14_5(XE32 *p, unsigned v) noexcept {
     set_le32(p, (get_le32(p) & 0xfff8001f) | ((v & 0x00003fff) << 5));
 }
 
@@ -289,17 +304,21 @@ inline void set_le14_5(void *p, unsigned v) noexcept {
 **************************************************************************/
 
 static forceinline constexpr int sign_extend(unsigned v, unsigned bits) noexcept {
+#if (ACC_ARCH_M68K) // no barrel shifter
     const unsigned sign_bit = 1u << (bits - 1);
-    v &= sign_bit | (sign_bit - 1);
-    v |= 0u - (v & sign_bit);
-    return ACC_ICAST(int, v);
+    return ACC_ICAST(int, (v & (sign_bit - 1)) - (v & sign_bit));
+#else
+    return ACC_ICAST(int, v << (32 - bits)) >> (32 - bits);
+#endif
 }
 
 static forceinline constexpr upx_int64_t sign_extend(upx_uint64_t v, unsigned bits) noexcept {
+#if (ACC_ARCH_M68K) // no barrel shifter
     const upx_uint64_t sign_bit = 1ull << (bits - 1);
-    v &= sign_bit | (sign_bit - 1);
-    v |= 0ull - (v & sign_bit);
-    return ACC_ICAST(upx_int64_t, v);
+    return ACC_ICAST(upx_int64_t, (v & (sign_bit - 1)) - (v & sign_bit));
+#else
+    return ACC_ICAST(upx_int64_t, v << (64 - bits)) >> (64 - bits);
+#endif
 }
 
 REQUIRE_XE16
@@ -714,10 +733,27 @@ T *operator+(T *ptr, const LE64 &v) noexcept DELETED_FUNCTION;
 template <class T>
 T *operator-(T *ptr, const LE64 &v) noexcept DELETED_FUNCTION;
 
+#if !ALLOW_INT_PLUS_MEMBUFFER
+template <class T>
+T *operator+(const BE16 &v, T *ptr) noexcept DELETED_FUNCTION;
+template <class T>
+T *operator+(const BE32 &v, T *ptr) noexcept DELETED_FUNCTION;
+template <class T>
+T *operator+(const LE16 &v, T *ptr) noexcept DELETED_FUNCTION;
+template <class T>
+T *operator+(const LE32 &v, T *ptr) noexcept DELETED_FUNCTION;
+#endif // ALLOW_INT_PLUS_MEMBUFFER
+
+template <class T>
+T *operator+(const BE64 &v, T *ptr) noexcept DELETED_FUNCTION;
+template <class T>
+T *operator+(const LE64 &v, T *ptr) noexcept DELETED_FUNCTION;
+
 /*************************************************************************
 // global overloads
 **************************************************************************/
 
+// TODO later: move these to upx namespace in util/cxxlib.h; see conf.h
 inline unsigned ALIGN_DOWN(unsigned a, const BE32 &b) { return ALIGN_DOWN(a, unsigned(b)); }
 inline unsigned ALIGN_DOWN(const BE32 &a, unsigned b) { return ALIGN_DOWN(unsigned(a), b); }
 inline unsigned ALIGN_UP(unsigned a, const BE32 &b) { return ALIGN_UP(a, unsigned(b)); }
@@ -728,7 +764,7 @@ inline unsigned ALIGN_DOWN(const LE32 &a, unsigned b) { return ALIGN_DOWN(unsign
 inline unsigned ALIGN_UP(unsigned a, const LE32 &b) { return ALIGN_UP(a, unsigned(b)); }
 inline unsigned ALIGN_UP(const LE32 &a, unsigned b) { return ALIGN_UP(unsigned(a), b); }
 
-// TODO: introduce upx::umax() and upx::umin()
+// TODO later: introduce upx::umax() and upx::umin()
 inline unsigned UPX_MAX(unsigned a, const BE16 &b) { return UPX_MAX(a, unsigned(b)); }
 inline unsigned UPX_MAX(const BE16 &a, unsigned b) { return UPX_MAX(unsigned(a), b); }
 inline unsigned UPX_MIN(unsigned a, const BE16 &b) { return UPX_MIN(a, unsigned(b)); }
@@ -753,6 +789,26 @@ inline unsigned UPX_MIN(const LE32 &a, unsigned b) { return UPX_MIN(unsigned(a),
 // misc support
 **************************************************************************/
 
+// for use with qsort()
+extern "C" {
+int __acc_cdecl_qsort be16_compare(const void *, const void *);
+int __acc_cdecl_qsort be24_compare(const void *, const void *);
+int __acc_cdecl_qsort be32_compare(const void *, const void *);
+int __acc_cdecl_qsort be64_compare(const void *, const void *);
+int __acc_cdecl_qsort le16_compare(const void *, const void *);
+int __acc_cdecl_qsort le24_compare(const void *, const void *);
+int __acc_cdecl_qsort le32_compare(const void *, const void *);
+int __acc_cdecl_qsort le64_compare(const void *, const void *);
+int __acc_cdecl_qsort be16_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort be24_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort be32_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort be64_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort le16_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort le24_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort le32_compare_signed(const void *, const void *);
+int __acc_cdecl_qsort le64_compare_signed(const void *, const void *);
+} // extern "C"
+
 // <type_traits> upx_is_integral; see conf.h
 #define TT_UPX_IS_INTEGRAL(T)                                                                      \
     template <>                                                                                    \
@@ -794,26 +850,6 @@ typedef LE64 NE64;
 #define ne64_compare_signed le64_compare_signed
 #endif
 
-// for use with qsort()
-extern "C" {
-int __acc_cdecl_qsort be16_compare(const void *, const void *);
-int __acc_cdecl_qsort be24_compare(const void *, const void *);
-int __acc_cdecl_qsort be32_compare(const void *, const void *);
-int __acc_cdecl_qsort be64_compare(const void *, const void *);
-int __acc_cdecl_qsort le16_compare(const void *, const void *);
-int __acc_cdecl_qsort le24_compare(const void *, const void *);
-int __acc_cdecl_qsort le32_compare(const void *, const void *);
-int __acc_cdecl_qsort le64_compare(const void *, const void *);
-int __acc_cdecl_qsort be16_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort be24_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort be32_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort be64_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort le16_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort le24_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort le32_compare_signed(const void *, const void *);
-int __acc_cdecl_qsort le64_compare_signed(const void *, const void *);
-} // extern "C"
-
 /*************************************************************************
 // Provide namespaces and classes to abstract endianness policies.
 //
