@@ -27,6 +27,8 @@
 // lots of tests (and probably quite a number of redundant tests)
 // modern compilers will optimize away much of this code
 
+#include "../util/system_headers.h"
+#include <vector> // std::vector
 #include "../conf.h"
 
 /*************************************************************************
@@ -49,6 +51,12 @@ ACC_COMPILE_TIME_ASSERT_HEADER((!upx::is_same_any_v<int, char>) )
 ACC_COMPILE_TIME_ASSERT_HEADER((!upx::is_same_any_v<int, char, char>) )
 ACC_COMPILE_TIME_ASSERT_HEADER((!upx::is_same_any_v<int, char, long>) )
 
+ACC_COMPILE_TIME_ASSERT_HEADER((upx::is_same_any_v<ptrdiff_t, int, long, long long>) )
+ACC_COMPILE_TIME_ASSERT_HEADER(
+    (upx::is_same_any_v<size_t, unsigned, unsigned long, unsigned long long>) )
+ACC_COMPILE_TIME_ASSERT_HEADER(
+    (upx::is_same_any_v<upx_uintptr_t, unsigned, unsigned long, unsigned long long>) )
+
 ACC_COMPILE_TIME_ASSERT_HEADER(usizeof(int) == sizeof(int))
 ACC_COMPILE_TIME_ASSERT_HEADER(usizeof('a') == sizeof(char))
 ACC_COMPILE_TIME_ASSERT_HEADER(usizeof("") == 1)
@@ -97,6 +105,30 @@ ACC_COMPILE_TIME_ASSERT_HEADER(!compile_time::string_gt("abc", "abz"))
 ACC_COMPILE_TIME_ASSERT_HEADER(!compile_time::string_ge("abc", "abz"))
 ACC_COMPILE_TIME_ASSERT_HEADER(compile_time::string_le("abc", "abz"))
 
+ACC_COMPILE_TIME_ASSERT_HEADER(compile_time::mem_eq((const char *) nullptr, nullptr, 0))
+ACC_COMPILE_TIME_ASSERT_HEADER(compile_time::mem_eq((const byte *) nullptr, nullptr, 0))
+ACC_COMPILE_TIME_ASSERT_HEADER(compile_time::mem_eq("", "", 0))
+ACC_COMPILE_TIME_ASSERT_HEADER(compile_time::mem_eq("abc", "abc", 3))
+ACC_COMPILE_TIME_ASSERT_HEADER(!compile_time::mem_eq("abc", "abz", 3))
+
+/*************************************************************************
+//
+**************************************************************************/
+
+TEST_CASE("std::vector") {
+    constexpr size_t N = 16;
+    std::vector<int> v(N);
+    CHECK(v.end() - v.begin() == N);
+    CHECK(&v[0] == &(*(v.begin())));
+    // CHECK(&v[0] + N == &(*(v.end()))); // TODO later: is this legal??
+    // TODO later: make sure that this throws
+#if defined(_LIBCPP_HARDENING_MODE_DEBUG) &&                                                       \
+    (_LIBCPP_HARDENING_MODE == _LIBCPP_HARDENING_MODE_DEBUG)
+    CHECK_THROWS((void) &v[N]);
+#endif
+    UNUSED(v);
+}
+
 /*************************************************************************
 // UPX_CXX_DISABLE_xxx
 **************************************************************************/
@@ -134,7 +166,7 @@ struct MyVType2 {
     UPX_CXX_DISABLE_ADDRESS(Self)
     UPX_CXX_DISABLE_NEW_DELETE(Self)
 };
-TEST_CASE("upx_cxx_disable") {
+TEST_CASE("UPX_CXX_DISABLE_xxx") {
     MyType1<char, int, 1> dummy1;
     MyType2<char, int, 2> dummy2;
     MyVType1<char, int, 1> vdummy1;
@@ -218,8 +250,114 @@ struct Z2_X2 : public X2 {
 // util
 **************************************************************************/
 
-TEST_CASE("ptr_static_cast") {
-    // check that we don't trigger any -Wcast-align warnings
+#if WITH_THREADS
+TEST_CASE("upx::ptr_std_atomic_cast") {
+    // pointer-size
+    CHECK_EQ(upx::ptr_std_atomic_cast((void **) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((uintptr_t *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((upx_uintptr_t *) nullptr), nullptr);
+#if 1
+    // more fundamental types
+    CHECK_EQ(upx::ptr_std_atomic_cast((char *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((short *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((int *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((long *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((ptrdiff_t *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((size_t *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((upx_int8_t *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((upx_int16_t *) nullptr), nullptr);
+    CHECK_EQ(upx::ptr_std_atomic_cast((upx_int32_t *) nullptr), nullptr);
+#endif
+}
+#endif
+
+TEST_CASE("upx::atomic_exchange") {
+    {
+        upx_uintptr_t x = (upx_uintptr_t) 0 - 1;
+        upx_uintptr_t y = upx::atomic_exchange(&x, (upx_uintptr_t) 2);
+        CHECK_EQ(x, 2);
+        CHECK_EQ(y, (upx_uintptr_t) 0 - 1);
+        UNUSED(y);
+    }
+    {
+        const int buf[2] = {101, 202};
+        const int *ptr_array[2] = {&buf[0], &buf[1]};
+        assert_noexcept(*ptr_array[0] == 101 && *ptr_array[1] == 202);
+        const int *p = upx::atomic_exchange(&ptr_array[0], ptr_array[1]);
+        CHECK_EQ(p, buf + 0);
+        assert_noexcept(*ptr_array[0] == 202 && *ptr_array[1] == 202);
+        p = upx::atomic_exchange(&ptr_array[1], p);
+        CHECK_EQ(p, buf + 1);
+        assert_noexcept(*ptr_array[0] == 202 && *ptr_array[1] == 101);
+        UNUSED(p);
+    }
+}
+
+TEST_CASE("upx::ObjectDeleter 1") {
+    LE16 *o = nullptr; // object
+    LE32 *a = nullptr; // array
+    LE64 *m = nullptr; // malloc
+    {
+        auto o_deleter = upx::ObjectDeleter(&o, 1);
+        o = new LE16;
+        assert(o != nullptr);
+        auto a_deleter = upx::ArrayDeleter(&a, 1);
+        a = New(LE32, 1);
+        assert(a != nullptr);
+        auto m_deleter = upx::MallocDeleter(&m, 1);
+        m = (LE64 *) ::malloc(sizeof(LE64));
+        assert(m != nullptr);
+    }
+    assert(o == nullptr);
+    assert(a == nullptr);
+    assert(m == nullptr);
+    // test "const" versions
+    {
+        const auto o_deleter = upx::ObjectDeleter(&o, 1);
+        o = new LE16;
+        assert(o != nullptr);
+        const auto a_deleter = upx::ArrayDeleter(&a, 1);
+        a = New(LE32, 1);
+        assert(a != nullptr);
+        const auto m_deleter = upx::MallocDeleter(&m, 1);
+        m = (LE64 *) ::malloc(sizeof(LE64));
+        assert(m != nullptr);
+    }
+    assert(o == nullptr);
+    assert(a == nullptr);
+    assert(m == nullptr);
+}
+
+TEST_CASE("upx::ObjectDeleter 2") {
+    constexpr size_t N = 2;
+    BE16 *o[N]; // multiple objects
+    BE32 *a[N]; // multiple arrays
+    BE64 *m[N]; // multiple mallocs
+    {
+        auto o_deleter = upx::ObjectDeleter(o, 0);
+        auto a_deleter = upx::ArrayDeleter(a, 0);
+        auto m_deleter = upx::MallocDeleter(m, 0);
+        for (size_t i = 0; i < N; i++) {
+            o[i] = new BE16;
+            assert(o[i] != nullptr);
+            o_deleter.count += 1;
+            a[i] = New(BE32, 1 + i);
+            assert(a[i] != nullptr);
+            a_deleter.count += 1;
+            m[i] = (BE64 *) ::malloc(sizeof(BE64));
+            assert(m[i] != nullptr);
+            m_deleter.count += 1;
+        }
+    }
+    for (size_t i = 0; i < N; i++) {
+        assert(o[i] == nullptr);
+        assert(a[i] == nullptr);
+        assert(m[i] == nullptr);
+    }
+}
+
+TEST_CASE("upx::ptr_static_cast") {
+    // check that we do not trigger any -Wcast-align warnings
     using upx::ptr_static_cast;
     void *vp = nullptr;
     byte *bp = nullptr;
@@ -258,7 +396,7 @@ TEST_CASE("ptr_static_cast") {
     assert((ic == ptr_static_cast<const int *>(ic)));
 }
 
-TEST_CASE("noncopyable") {
+TEST_CASE("upx::noncopyable") {
     struct Test : private upx::noncopyable {
         int v = 1;
     };
@@ -282,7 +420,7 @@ TEST_CASE("noncopyable") {
 namespace {
 template <class T>
 struct TestTriBool {
-    static void test(bool expect_true) {
+    static noinline void test(bool expect_true) {
         static_assert(std::is_class<T>::value);
         static_assert(std::is_nothrow_default_constructible<T>::value);
         static_assert(std::is_nothrow_destructible<T>::value);
@@ -290,8 +428,12 @@ struct TestTriBool {
         static_assert(std::is_trivially_copyable<T>::value);
         static_assert(sizeof(typename T::value_type) == sizeof(typename T::underlying_type));
         static_assert(alignof(typename T::value_type) == alignof(typename T::underlying_type));
-#if (ACC_ARCH_M68K && ACC_OS_TOS && ACC_CC_GNUC) && defined(__MINT__)
+#if defined(__m68k__) && defined(__atarist__) && defined(__GNUC__)
         // broken compiler or broken ABI
+#elif defined(__i386__) && defined(__GNUC__) && (__GNUC__ == 7) && !defined(__clang__)
+        static_assert(sizeof(T) == sizeof(typename T::underlying_type));
+        // i386: "long long" enum align bug/ABI problem on older compilers
+        static_assert(alignof(T) <= alignof(typename T::underlying_type));
 #else
         static_assert(sizeof(T) == sizeof(typename T::underlying_type));
         static_assert(alignof(T) == alignof(typename T::underlying_type));
@@ -366,9 +508,11 @@ struct TestTriBool {
             assert(a);
             assert(bool(a));
             assert((a ? true : false));
+            assert((!a ? false : true));
         } else {
             assert(!a);
             assert(!bool(a));
+            assert((a ? false : true));
             assert((!a ? true : false));
         }
         assert(!a.isStrictFalse());
@@ -381,7 +525,7 @@ struct TestTriBool {
 };
 } // namespace
 
-TEST_CASE("TriBool") {
+TEST_CASE("upx::TriBool") {
     using upx::TriBool, upx::tribool;
     static_assert(!tribool(false));
     static_assert(tribool(true));
