@@ -382,9 +382,18 @@ PackLinuxI386::buildLinuxLoader(
         if (bele->isBE()) // big endian - bswap32
             properties = bswap32(properties);
         linker->defineSymbol("lzma_properties", properties);
-        // -2 for properties
-        linker->defineSymbol("lzma_c_len", ph.c_len - 2);
-        linker->defineSymbol("lzma_u_len", ph.u_len);
+
+        // These lengths assume only one block (typ. 524288 bytes: 0.5 MiB).
+        // i386 handles more than one block, and computes the lengths
+        // dynamically from struct b_info.  Why do others need these?
+        if (linker->findSymbol("lzma_c_len", false)) {
+            // -2 for properties
+            linker->defineSymbol("lzma_c_len", ph.c_len - 2);
+        }
+        if (linker->findSymbol("lzma_u_len", false)) {
+            linker->defineSymbol("lzma_u_len", ph.c_len);
+        }
+
         unsigned const stack = getDecompressorWrkmemSize();
         linker->defineSymbol("lzma_stack_adjust", 0u - stack);
     }
