@@ -24,7 +24,6 @@
    <jreiser@users.sourceforge.net>
  */
 
-
 /*************************************************************************
 // Use the preprocessor to work around
 //   - that the types embedding these enums have to be PODs, and
@@ -33,6 +32,10 @@
 //   - that older compilers do not correctly perform EBCO
 **************************************************************************/
 
+#if CLANG_FORMAT_DUMMY_CLASS
+class Dummy {
+#endif
+
 #ifdef WANT_MACH_HEADER_ENUM  /*{*/
 #undef WANT_MACH_HEADER_ENUM
     enum : unsigned { // magic
@@ -168,4 +171,8 @@
     };
 #endif  /*}*/
 
+#if CLANG_FORMAT_DUMMY_CLASS
+}; // class
+#endif
+
 /* vim:set ts=4 sw=4 et: */
