@@ -75,9 +75,7 @@ static const CLANG_FORMAT_DUMMY_STATEMENT
 **************************************************************************/
 
 PackPs1::PackPs1(InputFile *f)
-    : super(f), isCon(!opt->ps1_exe.boot_only), is32Bit(!opt->ps1_exe.do_8bit), buildPart2(0),
-      foundBss(0), sa_cnt(0), overlap(0), sz_lunc(0), sz_lcpr(0), pad_code(0), bss_start(0),
-      bss_end(0) {
+    : super(f), isCon(!opt->ps1_exe.boot_only), is32Bit(!opt->ps1_exe.do_8bit) {
     bele = &N_BELE_RTP::le_policy;
 
     COMPILE_TIME_ASSERT(sizeof(ps1_exe_t) == 136)
