@@ -227,6 +227,16 @@ typedef int f_expand(
           nrv_byte *dst, nrv_uint *dst_len, int method );
 
 
+void my_bkpt(int rv)
+{
+#if defined(__i386__)  //{
+    __asm__("int3");
+#endif  //}
+    (void)rv;
+}
+#define error1(rv) {my_bkpt(rv); goto error1;}
+#define error(rv)  {my_bkpt(rv); goto error;}
+
 /*************************************************************************
 // upx_main - called by our entry code
 //
@@ -292,7 +302,7 @@ void upx_main(
     // by checking the random program id. (The id is both stored
     // in the header and patched into this stub.)
     if (header.p_progid != UPX3)
-        goto error1;
+        error1(11);
 
 
     //
@@ -338,7 +348,7 @@ void upx_main(
     {
         int err = unlink(tmpname);
         if (err != -ENOENT && err != 0)
-            goto error1;
+            error1(12);
     }
 
     // Create the temporary output file.
@@ -350,12 +360,12 @@ void upx_main(
 #if 0
     // Save some bytes of code - the ftruncate() below will fail anyway.
     if (fdo < 0)
-        goto error;
+        error(13);
 #endif
 
     // Set expected uncompressed file size.
     if (ftruncate(fdo, header.p_filesize) != 0)
-        goto error;
+        error(14);
 
 
     //
@@ -364,10 +374,11 @@ void upx_main(
 
 #if (USE_MMAP_FO)
     // FIXME: packer could set length
-    buf = (unsigned char *)mmap(0, header.p_filesize,
+    // Ask for one page more to reserve placement.
+    buf = (unsigned char *)mmap(0, header.p_filesize -PAGE_MASK,
         PROT_READ | PROT_WRITE, MAP_SHARED, fdo, 0);
     if ((unsigned long) buf >= (unsigned long) -4095)
-        goto error;
+        error(15);
 
     // Decompressor can overrun the output by 3 bytes.
     // Defend against SIGSEGV by using a scratch page.
@@ -381,7 +392,7 @@ void upx_main(
     buf = mmap(0, (header.p_blocksize + OVERHEAD + ~PAGE_MASK) & PAGE_MASK,
         PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0 );
     if ((unsigned long) buf >= (unsigned long) -4095)
-        goto error;
+        error(16);
 #endif
 
     //
@@ -405,16 +416,16 @@ void upx_main(
         if (h.sz_unc == 0)                      // uncompressed size 0 -> EOF
         {
             if (h.sz_cpr != UPX_MAGIC_LE32)     // h.sz_cpr must be h->magic
-                goto error;
+                error(17);
             if (header.p_filesize != 0)         // all bytes must be written
-                goto error;
+                error(18);
             break;
         }
         //   Note: if sz_unc == sz_cpr then the block was not
         //   compressible and is stored in its uncompressed form.
 
         if (h.sz_cpr > h.sz_unc || h.sz_cpr > header.p_blocksize)
-            goto error;
+            error(19);
         // Now we have:
         //   assert(h.sz_cpr <= h.sz_unc);
         //   assert(h.sz_unc > 0 && h.sz_unc <= blocksize);
@@ -425,7 +436,7 @@ void upx_main(
             int i = (*f_decompress)((unsigned char *)xi.buf, h.sz_cpr, buf,
                 &out_len, *(int *)(void *)&h.b_method);
             if (i != 0 || out_len != (nrv_uint)h.sz_unc)
-                goto error;
+                error(20);
             // Right now, unfilter is combined with decompression.
             // (*f_unfilter)(buf, out_len, cto8);
             (void)f_unf;
@@ -450,7 +461,7 @@ void upx_main(
 #else
         // write output file
         if (xwrite(fdo, buf, h.sz_unc) != 0)
-            goto error;
+            error(21);
 #endif
 
         header.p_filesize -= h.sz_unc;
@@ -481,7 +492,7 @@ void upx_main(
 #endif
 
     if (close(fdo) != 0)
-        goto error;
+        error(22);
 
 
     //
@@ -492,7 +503,7 @@ void upx_main(
     // Jamie Lokier <nospam@cern.ch> for this nice idea.
 
     if (0 > go_self(tmpname, argv, envp))
-        goto error;
+        error(23);
 
 
     //
@@ -531,7 +542,7 @@ void upx_main(
     //
 
     // If we return from execve() there was an error. Give up.
-    goto error;
+    error(24);
 }
 
 /* vim:set ts=4 sw=4 et: */
