@@ -29,9 +29,12 @@
 // dispatch. PackMaster by itself will instantiate a concrete subclass
 // of class PackerBase which then does the actual work.
 // And see p_com.cpp for a simple executable format.
+//
+// And this file also has the burden to deal with all those pesky low-level
+// file handling issues.
 
 #define WANT_WINDOWS_LEAN_H 1 // _get_osfhandle, GetFileTime, SetFileTime
-#include "headers.h"
+#include "util/system_headers.h"
 #if USE_UTIMENSAT
 #include <sys/types.h>
 #include <fcntl.h>
@@ -44,7 +47,7 @@
 #include "util/membuffer.h"
 
 #if USE_UTIMENSAT && defined(AT_FDCWD)
-#elif (defined(_WIN32) || defined(__CYGWIN__)) && 1
+#elif defined(_WIN32) || defined(__CYGWIN__)
 #define USE_SETFILETIME 1
 #elif (ACC_OS_DOS32) && defined(__DJGPP__)
 #define USE_FTIME 1
@@ -325,7 +328,7 @@ void do_one_file(const char *const iname, char *const oname) may_throw {
         }
     }
 
-    // handle command - actual work is here
+    // handle command - actual work starts HERE
     PackMaster pm(&fi, opt);
     if (opt->cmd == CMD_COMPRESS)
         pm.pack(&fo);
